@@ -38,7 +38,6 @@ extern "C" {
 
 /* ------------------------------------------------------------------------ */
 /* flag */
-
 /* ------------------------------------------------------------------------ */
 /* @class Object Object knh_Object_t @Cyclic */
 /* @flag Object.Release!Debug OF (%s)->h.flag 'is:*:is:*' */
@@ -64,6 +63,11 @@ typedef struct knh_ObjectField_t {
 	size_t  bsize;
 } knh_ObjectField_t ;
 
+/* ------------------------------------------------------------------------ */
+
+#define KNH_CHECK_IMMUTABLE(ctx, o, L) \
+	if(knh_Object_isImmutable(o)) goto L;
+
 /* ------------------------------------------------------------------------ */
 /* @class Any Object knh_Any_t @Cyclic */
 /* @class This Any knh_Any_t @Cyclic  @Private */
@@ -197,6 +201,8 @@ typedef struct knh_Range_t {
 /* @class Array Object knh_Array_t @Cyclic @Param1 */
 /* @flag Array.Dim ARY:1 (%s)->h.flag 'is:set:*:*' */
 
+#define KNH_ARRAY_INITSIZE  (KNH_FASTMALLOC_SIZE / sizeof(knh_Object_t*))
+
 typedef struct knh_Array_t {
 	knh_hObject_t h;
 	struct knh_Object_t **list;
@@ -208,6 +214,8 @@ typedef struct knh_Array_t {
 /* @class IArray Object knh_IArray_t @Cyclic @Param1 */
 /* @flag IArray.Dim ARY:1 (%s)->h.flag 'is:set:*:*' */
 
+#define KNH_IARRAY_INITSIZE  (KONOHA_SMALLPAGESIZE / sizeof(knh_integer_t))
+
 typedef struct knh_IArray_t {
 	knh_hObject_t h;
 	knh_integer_t *ilist;
@@ -219,6 +227,8 @@ typedef struct knh_IArray_t {
 /* @class FArray Object knh_FArray_t @Cyclic @Param1 */
 /* @flag FArray.Dim ARY:1 (%s)->h.flag 'is:set:*:*' */
 
+#define KNH_FARRAY_INITSIZE  (KONOHA_SMALLPAGESIZE / sizeof(knh_float_t))
+
 typedef struct knh_FArray_t {
 	knh_hObject_t h;
 	knh_float_t *flist;
@@ -298,87 +308,55 @@ typedef struct knh_DictSet_t {
 } knh_DictSet_t;
 
 /* ------------------------------------------------------------------------ */
-/* @class HashMap Object knh_HashMap_struct @Param2 */
 
-typedef struct knh_hashmapentry_t {
+typedef struct knh_hashentry_t {
 	knh_hcode_t hcode;
+	struct knh_hashentry_t *next;
 	Object *key;
-	Object *value;
-	struct knh_hashmapentry_t *next;
-} knh_hashmapentry_t ;
+	union {
+		Object *value;
+		knh_uint_t uvalue;
+	};
+} knh_hashentry_t;
 
-typedef struct knh_HashMap {
-	size_t capacity;
+typedef knh_hashentry_t* (*knh_fhashentry_init)(Ctx *);
+typedef void (*knh_fhashentry_traverse)(Ctx *, knh_hashentry_t*, knh_ftraverse);
+
+typedef struct {
 	size_t size;
-	knh_hashmapentry_t** array;
-	knh_fstruct_compareTo fcompareTo;
-	size_t table_size;
-	knh_hashmapentry_t*  unused;
-	knh_hashmapentry_t*  table;
-	knh_hashmapentry_t** ex_tables;
-	size_t               ex_capacity;
-	size_t               ex_size;
-	char *DBG_name;
-} knh_HashMap_struct;
+	knh_fhashentry_init finit;
+	knh_fhashentry_traverse ftraverse;
+} knh_hash_op ;
 
 /* ------------------------------------------------------------------------ */
+/* @class HashMap Object knh_HashMap_struct @Param2 */
 /* @class HashSet Object knh_HashSet_struct @Param1 */
 
-typedef struct knh_hashsetentry_t {
-	knh_hcode_t hcode;
-	Object *key;
-	knh_uint_t value;
-	struct knh_hashsetentry_t *next;
-} knh_hashsetentry_t ;
+#define KNH_HASH_TABLESIZE  KONOHA_PAGESIZE
+#define KNH_HASHENTRY(T,ESIZE,i)  (knh_hashentry_t*)&((T)[(ESIZE) * (i)])
 
-typedef struct knh_HashSet {
+typedef struct {
+	knh_hashentry_t** array;
+	size_t hmax;
 	size_t capacity;
 	size_t size;
-	knh_hashsetentry_t** array;
-	knh_fstruct_compareTo fcompareTo;
-	size_t table_size;
-	knh_hashsetentry_t*  unused;
-	knh_hashsetentry_t*  table;
-	knh_hashsetentry_t** ex_tables;
-	size_t               ex_capacity;
-	size_t               ex_size;
+	knh_hashentry_t*  unused_first;
+	char** tables;
+	size_t tables_capacity;
+	size_t tables_size;
+	char*  tables_first;
 	char *DBG_name;
-} knh_HashSet_struct;
+} knh_Hash_struct;
 
-/* ------------------------------------------------------------------------ */
-/* @class LRUHashMap Object knh_LRUHashMap_struct */
+typedef knh_Hash_struct knh_HashMap_struct;
+typedef knh_Hash_struct knh_HashSet_struct;
 
-typedef struct knh_lruentry_t {
-	knh_hcode_t hcode;
-	Object *key;
-	Object *value;
-	struct knh_lruentry_t  *value_next;
-	struct knh_lruentry_t  *prev;
-	struct knh_lruentry_t  *next;
-} knh_lruentry_t ;
-
-typedef struct knh_LRUHashMap {
-	size_t table_size;
-	knh_lruentry_t*  table;
-	knh_lruentry_t*  unused;
-	knh_lruentry_t** array;
-	knh_lruentry_t *first;
-	knh_lruentry_t *last;
-	char *dbg_name;
-	size_t hitc;
-	size_t missc;
-} knh_LRUHashMap_struct;
-
-///* ------------------------------------------------------------------------ */
-///* class Map Object knh_Map */
-//
-//struct knh_Map;
-//typedef Object* (*f_map_get)(Ctx *, Object *b, Object *k);
-//typedef Object* (*f_map_set)(Ctx *, Object *b, Object *k, Object *v);
-//
-//typedef struct knh_Map {
-//	Object *map;
-//} knh_Map ;
+typedef struct {
+	knh_hObject_t h;
+	knh_Hash_struct *b;
+	knh_fstruct_compareTo fcmp;
+	knh_hash_op *hashop;
+} knh_Hash_t ;
 
 /* ------------------------------------------------------------------------ */
 /* @class DictIdx Object knh_DictIdx_t @Private */
