@@ -100,11 +100,6 @@ extern "C" {
 
 
 #define knh_Float_value(o)    (o)->value
-#define LRUMap                          HashMap
-#define knh_LRUMap                      knh_HashMap_struct
-#define new_LRUMap(ctx,n)               new_HashMap(ctx,n)
-#define knh_LRUMap_get(ctx,b,kh,ko)     knh_HashMap_get(ctx,b,kh,ko)
-#define knh_LRUMap_set(ctx,b,kh,ko,v)   knh_HashMap_set(ctx,b,kh,ko,v)
 #define knh_Int_value(o)      (o)->value
 #define knh_Method_mn(mtd)    DP(mtd)->mn
 #define knh_Method_mf(mtd)    DP(mtd)->mf
