@@ -43,11 +43,25 @@ extern "C" {
 METHOD knh__Array_new(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_new__array(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_new__init(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_new(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_new__array(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_new__init(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_new(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_new__array(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_new__init(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_add(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opLshift(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_add(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_opLshift(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_add(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_opLshift(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opSize(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_get(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_get(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_get(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_set(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_set(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_set(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_setAll(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opSubset(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opSubsete(Ctx *ctx, knh_sfp_t *sfp);
@@ -63,6 +77,14 @@ METHOD knh__Array_sort(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_reverse(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_swap(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_shuffle(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_sort(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_reverse(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_swap(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_shuffle(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_sort(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_reverse(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_swap(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_shuffle(Ctx *ctx, knh_sfp_t *sfp);
 void knh_Array__k(Ctx *ctx, Array *o, OutputStream *w, String *m);
 MAPPER knh_Array_Iterator(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_opItr(Ctx *ctx, knh_sfp_t *sfp);
@@ -76,6 +98,18 @@ METHOD knh__Array_get2D(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_get3D(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_set2D(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Array_set3D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_new__array2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_new__array3D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_get2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_get3D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_set2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__IArray_set3D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_new__array2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_new__array3D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_get2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_get3D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_set2D(Ctx *ctx, knh_sfp_t *sfp);
+METHOD knh__FArray_set3D(Ctx *ctx, knh_sfp_t *sfp);
 /* ../src/api/bytes.c */
 METHOD knh__Bytes_new(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__Bytes_putc(Ctx *ctx, knh_sfp_t *sfp);
@@ -129,7 +163,7 @@ METHOD knh__HashMap_get(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__HashMap_opHas(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__HashMap_remove(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__HashMap_set(Ctx *ctx, knh_sfp_t *sfp);
-void knh_HashMap__k(Ctx *ctx, HashMap *o, OutputStream *w, String *m);
+void knh_HashMap__k(Ctx *ctx, knh_Hash_t *o, OutputStream *w, String *m);
 void knh_HashMap__dump(Ctx *ctx, HashMap *o, OutputStream *w, String *m);
 MAPPER knh_HashMap_Iterator(Ctx *ctx, knh_sfp_t *sfp);
 METHOD knh__HashMap_opItr(Ctx *ctx, knh_sfp_t *sfp);
@@ -312,8 +346,6 @@ AffineConv *new_AffineConv(Ctx *ctx, knh_float_t fa, knh_float_t fb);
 size_t knh_array_newsize(size_t newsize, size_t usize);
 size_t knh_array_index(Ctx *ctx, knh_int_t n, size_t size);
 void knh_Array_grow(Ctx *ctx, Array *a, size_t newsize, Object *v);
-void knh_IArray_grow(Ctx *ctx, IArray *a, size_t newsize, knh_integer_t v);
-void knh_FArray_grow(Ctx *ctx, FArray *a, size_t newsize, knh_float_t v);
 knh_Array_t* new_Array0(Ctx *ctx, size_t capacity);
 Object** knh_Array_list(Array *o);
 Object* knh_Array_last(Array *o);
@@ -457,20 +489,16 @@ FloatUnit *new_FloatUnit(Ctx *ctx, knh_flag_t flag, knh_class_t cid, String *tag
 Float* new_Float(Ctx *ctx, knh_float_t value);
 Float* new_FloatX__fast(Ctx *ctx, knh_class_t cid, knh_float_t value);
 Float* new_FloatX(Ctx *ctx, knh_class_t cid, knh_float_t value);
-/* ../src/class/knh_HashMap.c */
-knh_hashmapentry_t *new_hashmapentry(Ctx *ctx, HashMap *o);
-size_t knh_hashmapentry_capacity(Ctx *ctx, HashMap *o);
-knh_hashmapentry_t *knh_hashmapentry_at(HashMap *o, size_t n);
+/* ../src/class/knh_Hash.c */
+knh_hashentry_t *new_hashentry(Ctx *ctx, knh_Hash_t *o);
+void knh_hashentry_collect(Ctx *ctx, knh_Hash_t *o, knh_hashentry_t *e);
+knh_hashentry_t *knh_hashentry_at(knh_Hash_t *o, size_t n);
 HashMap* new_HashMap(Ctx *ctx, char *name, size_t capacity);
-Object *knh_HashMap_get__hcode(Ctx *ctx, HashMap *o, knh_hcode_t hcode);
+knh_hashentry_t *knh_Hash_get__hcode(Ctx *ctx, knh_Hash_t *o, knh_hcode_t hcode);
+void knh_Hash_remove__hcode(Ctx *ctx, knh_Hash_t *o, knh_hcode_t hcode);
+void knh_Hash_rehash(Ctx *ctx, knh_Hash_t *o);
 void knh_HashMap_set__hcode(Ctx *ctx, HashMap *o, knh_hcode_t hcode, Any *value);
-void knh_HashMap_remove__hcode(Ctx *ctx, HashMap *o, knh_hcode_t hcode);
-/* ../src/class/knh_HashSet.c */
-HashSet* new_HashSet(Ctx *ctx, char *name, size_t capacity);
-knh_uint_t knh_HashSet_get__hcode(Ctx *ctx, HashSet *o, knh_hcode_t hcode);
-void knh_HashSet_set__hcode(Ctx *ctx, HashSet *o, knh_hcode_t hcode, knh_uint_t value);
-void knh_HashSet_add__hcode(Ctx *ctx, HashSet *o, knh_hcode_t hcode);
-void knh_HashSet_remove__hcode(Ctx *ctx, HashSet *o, knh_hcode_t hcode);
+void knh_HashSet_set__hcode(Ctx *ctx, HashSet *o, knh_hcode_t hcode, knh_uint_t uvalue);
 /* ../src/class/knh_InputStream.c */
 Object *knh_InputStream_open(Ctx *ctx, InputStream *o, String *urn);
 int knh_InputStream_getc(Ctx *ctx, InputStream *o);
@@ -492,11 +520,6 @@ void knh_IntUnit_initId(Ctx *ctx, IntUnit *b, knh_flag_t flag, knh_class_t cid,
 /* ../src/class/knh_Iterator.c */
 void knh_Iterator_close(Ctx *ctx, Iterator *it);
 Iterator* new_Iterator(Ctx *ctx, knh_class_t p1, Any *source, knh_fitrnext fnext);
-/* ../src/class/knh_LRUHashMap.c */
-LRUHashMap* new_LRUHashMap(Ctx *ctx, char *name, size_t capacity);
-Object *knh_LRUHashMap_get__hcode(Ctx *ctx, LRUHashMap *o, knh_hcode_t hcode);
-void knh_LRUHashMap_set__hcode(Ctx *ctx, LRUHashMap *o, knh_hcode_t hcode, Any *value);
-void knh_LRUHashMap_clear(Ctx *ctx, LRUHashMap *o);
 /* ../src/class/knh_Mapper.c */
 Mapper* new_Mapper(Ctx *ctx, knh_flag_t flag, knh_class_t scid, knh_class_t tcid, knh_fmapper fmap, Object *mapdata);
 Mapper* new_MapMap(Ctx *ctx, Mapper *m1, Mapper *m2);
@@ -661,6 +684,12 @@ char *knh_Method_file(Ctx *ctx, Method *mtd);
 int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 int knh_Asm_constId(Ctx *ctx, Asm *abr, Object *o);
 int knh_Asm_classId(Ctx *ctx, Asm *abr, knh_class_t cid);
+void knh_Int__const(Ctx *ctx, Int *o, OutputStream *w, String *m);
+void knh_Method__const(Ctx *ctx, Method *o, OutputStream *w, String *m);
+void knh_Method__c(Ctx *ctx, Method *mtd, OutputStream *w, String *m);
+void knh_Asm_genMethod(Ctx *ctx, Asm *abr, OutputStream *w);
+void knh_Asm_loadCompiledMethod(Ctx *ctx, Asm *abr);
+knh_fmethod knh_Asm_getCompiledMethod(Ctx *ctx, Asm *abr, knh_bytes_t cname, knh_bytes_t mname);
 /* ../src/compiler/konohac.c */
 int konohac_compile(Ctx *ctx, String *nsname, Stmt *stmt_head, int isEval);
 void konohac_eval(Ctx *ctx, String *nsname, InputStream *in);
