@@ -86,7 +86,6 @@ static char* StructName[] =  {
 	"DictSet",
 	"HashMap",
 	"HashSet",
-	"LRUHashMap",
 	"DictIdx",
 	"Class",
 	"ClassStruct",
@@ -289,15 +288,6 @@ static knh_StructData_t StructData[] = {
 		(knh_fstruct_hashCode)knh_HashSet_hashCode,
 		(knh_fstruct_newClass)knh_HashSet_newClass
 	},
-	{
-		"LRUHashMap", STRUCT_LRUHashMap, FLAG_LRUHashMap, sizeof(knh_LRUHashMap_struct),
-		(knh_fstruct_init)knh_LRUHashMap_init,
-		(knh_fstruct_copy)knh_LRUHashMap_copy, 
-		(knh_fstruct_traverse)knh_LRUHashMap_traverse,
-		(knh_fstruct_compareTo)knh_LRUHashMap_compareTo,
-		(knh_fstruct_hashCode)knh_LRUHashMap_hashCode,
-		(knh_fstruct_newClass)knh_LRUHashMap_newClass
-	},
 	{
 		"DictIdx", STRUCT_DictIdx, FLAG_DictIdx, 0,
 		(knh_fstruct_init)knh_DictIdx_init,
@@ -564,7 +554,7 @@ static knh_ClassData_t ClassData[] = {
 	{"konoha.Nue", CLASS_Nue, CFLAG_Nue, STRUCT_Nue, CLASS_Object, 0, 2, 0, knh_Nue_fdefault},
 	{"konoha.Boolean", CLASS_Boolean, CFLAG_Boolean, STRUCT_Boolean, CLASS_Object, 0, 3, 0, knh_Boolean_fdefault},
 	{"konoha.Number", CLASS_Number, CFLAG_Number, STRUCT_Number, CLASS_Object, 0, 0, 0, knh_Number_fdefault},
-	{"konoha.Int", CLASS_Int, CFLAG_Int, STRUCT_Int, CLASS_Number, 0, 29, 0, knh_Int_fdefault},
+	{"konoha.Int", CLASS_Int, CFLAG_Int, STRUCT_Int, CLASS_Number, 0, 30, 0, knh_Int_fdefault},
 	{"konoha.IntX", CLASS_IntX, CFLAG_IntX, STRUCT_IntX, CLASS_Number, 0, 0, 0, knh_IntX_fdefault},
 	{"konoha.Float", CLASS_Float, CFLAG_Float, STRUCT_Float, CLASS_Number, 0, 20, 0, knh_Float_fdefault},
 	{"konoha.FloatX", CLASS_FloatX, CFLAG_FloatX, STRUCT_FloatX, CLASS_Number, 0, 0, 0, knh_FloatX_fdefault},
@@ -573,20 +563,19 @@ static knh_ClassData_t ClassData[] = {
 	{"konoha.Bytes", CLASS_Bytes, CFLAG_Bytes, STRUCT_Bytes, CLASS_Object, 0, 9, 0, knh_Bytes_fdefault},
 	{"konoha.Tuple2", CLASS_Tuple2, CFLAG_Tuple2, STRUCT_Tuple2, CLASS_Object, 0, 0, 0, knh_Tuple2_fdefault},
 	{"konoha.Range", CLASS_Range, CFLAG_Range, STRUCT_Range, CLASS_Object, 0, 0, 0, knh_Range_fdefault},
-	{"konoha.Array", CLASS_Array, CFLAG_Array, STRUCT_Array, CLASS_Object, 0, 31, 0, knh_Array_fdefault},
-	{"konoha.IArray", CLASS_IArray, CFLAG_IArray, STRUCT_IArray, CLASS_Object, 0, 0, 0, knh_IArray_fdefault},
-	{"konoha.FArray", CLASS_FArray, CFLAG_FArray, STRUCT_FArray, CLASS_Object, 0, 0, 0, knh_FArray_fdefault},
+	{"konoha.Array", CLASS_Array, CFLAG_Array, STRUCT_Array, CLASS_Object, 0, 30, 0, knh_Array_fdefault},
+	{"konoha.IArray", CLASS_IArray, CFLAG_IArray, STRUCT_IArray, CLASS_Object, 0, 17, 0, knh_IArray_fdefault},
+	{"konoha.FArray", CLASS_FArray, CFLAG_FArray, STRUCT_FArray, CLASS_Object, 0, 17, 0, knh_FArray_fdefault},
 	{"konoha.Iterator", CLASS_Iterator, CFLAG_Iterator, STRUCT_Iterator, CLASS_Object, 0, 2, 0, knh_Iterator_fdefault},
 	{"konoha.DictMap", CLASS_DictMap, CFLAG_DictMap, STRUCT_DictMap, CLASS_Object, 0, 12, 0, knh_DictMap_fdefault},
 	{"konoha.DictSet", CLASS_DictSet, CFLAG_DictSet, STRUCT_DictSet, CLASS_Object, 0, 13, 0, knh_DictSet_fdefault},
 	{"konoha.HashMap", CLASS_HashMap, CFLAG_HashMap, STRUCT_HashMap, CLASS_Object, 0, 8, 0, knh_HashMap_fdefault},
 	{"konoha.HashSet", CLASS_HashSet, CFLAG_HashSet, STRUCT_HashSet, CLASS_Object, 0, 0, 0, knh_HashSet_fdefault},
-	{"konoha.LRUHashMap", CLASS_LRUHashMap, CFLAG_LRUHashMap, STRUCT_LRUHashMap, CLASS_Object, 0, 0, 0, knh_LRUHashMap_fdefault},
 	{"konoha.DictIdx", CLASS_DictIdx, CFLAG_DictIdx, STRUCT_DictIdx, CLASS_Object, 0, 6, 0, knh_DictIdx_fdefault},
 	{"konoha.Class", CLASS_Class, CFLAG_Class, STRUCT_Class, CLASS_Object, 0, 12, 0, knh_Class_fdefault},
 	{"konoha.ClassStruct", CLASS_ClassStruct, CFLAG_ClassStruct, STRUCT_ClassStruct, CLASS_Object, 0, 0, 0, knh_ClassStruct_fdefault},
 	{"konoha.MethodField", CLASS_MethodField, CFLAG_MethodField, STRUCT_MethodField, CLASS_Object, 0, 0, 0, knh_MethodField_fdefault},
-	{"konoha.Method", CLASS_Method, CFLAG_Method, STRUCT_Method, CLASS_Object, 0, 15, 0, knh_Method_fdefault},
+	{"konoha.Method", CLASS_Method, CFLAG_Method, STRUCT_Method, CLASS_Object, 0, 17, 0, knh_Method_fdefault},
 	{"konoha.Mapper", CLASS_Mapper, CFLAG_Mapper, STRUCT_Mapper, CLASS_Object, 0, 12, 0, knh_Mapper_fdefault},
 	{"konoha.ClassMap", CLASS_ClassMap, CFLAG_ClassMap, STRUCT_ClassMap, CLASS_Object, 0, 1, 0, knh_ClassMap_fdefault},
 	{"konoha.Closure", CLASS_Closure, CFLAG_Closure, STRUCT_Closure, CLASS_Object, 0, 3, 0, knh_Closure_fdefault},
@@ -679,6 +668,7 @@ static knh_FieldNameData_t FieldNameData[] = {
 	{"close", FIELDN_close},
 	{"compareTo", FIELDN_compareTo},
 	{"concat", FIELDN_concat},
+	{"const", FIELDN_const},
 	{"copy", FIELDN_copy},
 	{"d", FIELDN_d},
 	{"debug", FIELDN_debug},
@@ -884,19 +874,30 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_void_OutputStream_Any, 2+ 0, TYPE_void, TYPE_OutputStream, FIELDN_w, TYPE_Any, FIELDN_o},
 	{MFN_Any, 0+ 0, TYPE_Any},
 	{MFN_nThis_Int, 1+ 0, NNTYPE_This, TYPE_Int, FIELDN_i},
-	{MFN_nThis_Any1_, 1+ 0, NNTYPE_This, TYPE_Any1, FIELDN_v},
-	{MFN_void_Any1, 1+ 0, TYPE_void, TYPE_Any1, FIELDN_v},
-	{MFN_void_Any1_, 1+ 0, TYPE_void, TYPE_Any1, FIELDN_v},
+	{MFN_nThis_nInt, 1+ 0, NNTYPE_This, NNTYPE_Int, FIELDN_n},
+	{MFN_nThis_nAny1_, 1+ 0, NNTYPE_This, NNTYPE_Any1, FIELDN_v},
+	{MFN_nThis_nInt_, 1+ 0, NNTYPE_This, NNTYPE_Int, FIELDN_v},
+	{MFN_nThis_nFloat_, 1+ 0, NNTYPE_This, NNTYPE_Float, FIELDN_v},
+	{MFN_void_nAny1, 1+ 0, TYPE_void, NNTYPE_Any1, FIELDN_v},
+	{MFN_void_nAny1_, 1+ 0, TYPE_void, NNTYPE_Any1, FIELDN_v},
+	{MFN_void_nInt, 1+ 0, TYPE_void, NNTYPE_Int, FIELDN_v},
+	{MFN_void_nInt_, 1+ 0, TYPE_void, NNTYPE_Int, FIELDN_v},
+	{MFN_void_nFloat, 1+ 0, TYPE_void, NNTYPE_Float, FIELDN_v},
+	{MFN_void_nFloat_, 1+ 0, TYPE_void, NNTYPE_Float, FIELDN_v},
 	{MFN_nInt, 0+ 0, NNTYPE_Int},
-	{MFN_Any1_nInt, 1+ 0, TYPE_Any1, NNTYPE_Int, FIELDN_n},
-	{MFN_void_nInt_Any1, 2+ 0, TYPE_void, NNTYPE_Int, FIELDN_n, TYPE_Any1, FIELDN_v},
+	{MFN_nAny1_nInt, 1+ 0, NNTYPE_Any1, NNTYPE_Int, FIELDN_n},
+	{MFN_nInt_nInt, 1+ 0, NNTYPE_Int, NNTYPE_Int, FIELDN_n},
+	{MFN_nFloat_nInt, 1+ 0, NNTYPE_Float, NNTYPE_Int, FIELDN_n},
+	{MFN_void_nInt_nAny1, 2+ 0, TYPE_void, NNTYPE_Int, FIELDN_n, NNTYPE_Any1, FIELDN_v},
+	{MFN_void_nInt_nInt, 2+ 0, TYPE_void, NNTYPE_Int, FIELDN_n, NNTYPE_Int, FIELDN_v},
+	{MFN_void_nInt_nFloat, 2+ 0, TYPE_void, NNTYPE_Int, FIELDN_n, NNTYPE_Float, FIELDN_v},
+	{MFN_void_Any1, 1+ 0, TYPE_void, TYPE_Any1, FIELDN_v},
 	{MFN_nThis_Int_Int, 2+ 0, NNTYPE_This, TYPE_Int, FIELDN_s, TYPE_Int, FIELDN_e},
-	{MFN_void_nInt, 1+ 0, TYPE_void, NNTYPE_Int, FIELDN_n},
+	{MFN_void_nInt_Any1, 2+ 0, TYPE_void, NNTYPE_Int, FIELDN_n, TYPE_Any1, FIELDN_v},
 	{MFN_Any1, 0+ 0, TYPE_Any1},
 	{MFN_void, 0+ 0, TYPE_void},
 	{MFN_nInt_Any1, 1+ 0, NNTYPE_Int, TYPE_Any1, FIELDN_v},
 	{MFN_nBoolean_Any1, 1+ 0, NNTYPE_Boolean, TYPE_Any1, FIELDN_v},
-	{MFN_void_nInt_nInt, 2+ 0, TYPE_void, NNTYPE_Int, FIELDN_m, NNTYPE_Int, FIELDN_n},
 	{MFN_iAny1, 0+ 0, NNITYPE_Any1},
 	{MFN_nThis_nInt_nInt, 2+ 0, NNTYPE_This, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y},
 	{MFN_nThis_nInt_nInt_nInt, 3+ 0, NNTYPE_This, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z},
@@ -904,10 +905,17 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_Any1_nInt_nInt_nInt, 3+ 0, TYPE_Any1, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z},
 	{MFN_void_nInt_nInt_Any1, 3+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, TYPE_Any1, FIELDN_v},
 	{MFN_void_nInt_nInt_nInt_Any1, 4+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z, TYPE_Any1, FIELDN_v},
+	{MFN_nInt_nInt_nInt, 2+ 0, NNTYPE_Int, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y},
+	{MFN_nInt_nInt_nInt_nInt, 3+ 0, NNTYPE_Int, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z},
+	{MFN_void_nInt_nInt_nInt, 3+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_v},
+	{MFN_void_nInt_nInt_nInt_nInt, 4+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z, NNTYPE_Int, FIELDN_v},
+	{MFN_nFloat_nInt_nInt, 2+ 0, NNTYPE_Float, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y},
+	{MFN_nFloat_nInt_nInt_nInt, 3+ 0, NNTYPE_Float, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z},
+	{MFN_void_nInt_nInt_nFloat, 3+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Float, FIELDN_v},
+	{MFN_void_nInt_nInt_nInt_nFloat, 4+ 0, TYPE_void, NNTYPE_Int, FIELDN_x, NNTYPE_Int, FIELDN_y, NNTYPE_Int, FIELDN_z, NNTYPE_Float, FIELDN_v},
 	{MFN_nBytes_Int, 1+ 0, NNTYPE_Bytes, TYPE_Int, FIELDN_i},
 	{MFN_void_nBytes_Int_Int, 3+ 0, TYPE_void, NNTYPE_Bytes, FIELDN_b, TYPE_Int, FIELDN_n, TYPE_Int, FIELDN_l},
-	{MFN_void_nInt_, 1+ 0, TYPE_void, NNTYPE_Int, FIELDN_v},
-	{MFN_nInt_nInt, 1+ 0, NNTYPE_Int, NNTYPE_Int, FIELDN_n},
+	{MFN_nThis_Any1_, 1+ 0, NNTYPE_This, TYPE_Any1, FIELDN_v},
 	{MFN_Any1_nString, 1+ 0, TYPE_Any1, NNTYPE_String, FIELDN_k},
 	{MFN_nBoolean_nString, 1+ 0, NNTYPE_Boolean, NNTYPE_String, FIELDN_k},
 	{MFN_void_nString_Any1, 2+ 0, TYPE_void, NNTYPE_String, FIELDN_k, TYPE_Any1, FIELDN_v},
@@ -918,8 +926,6 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 	{MFN_void_nString_nInt, 2+ 0, TYPE_void, NNTYPE_String, FIELDN_k, NNTYPE_Int, FIELDN_v},
 	{MFN_void_String_, 1+ 0, TYPE_void, TYPE_String, FIELDN_v},
 	{MFN_nException_String_Any, 2+ 0, NNTYPE_Exception, TYPE_String, FIELDN_m, TYPE_Any, FIELDN_b},
-	{MFN_void_nAny1, 1+ 0, TYPE_void, NNTYPE_Any1, FIELDN_k},
-	{MFN_nBoolean_nAny1, 1+ 0, NNTYPE_Boolean, NNTYPE_Any1, FIELDN_k},
 	{MFN_void_nAny1_Any2, 2+ 0, TYPE_void, NNTYPE_Any1, FIELDN_k, TYPE_Any2, FIELDN_v},
 	{MFN_nConnection_nString, 1+ 0, NNTYPE_Connection, NNTYPE_String, FIELDN_u},
 	{MFN_nResultSet_nString, 1+ 0, NNTYPE_ResultSet, NNTYPE_String, FIELDN_q},
@@ -979,13 +985,27 @@ static knh_MethodFieldData_t MethodFieldData[] = {
 
 static knh_MethodData_t MethodData[] = {
 	{knh__Array_new, 0, CLASS_Array, METHODN_new, 0, MFN_nThis_Int, (void*)NULL},
-	{knh__Array_new__array, 0, CLASS_Array, METHODN_new__array, 0, MFN_nThis_Int, (void*)NULL},
-	{knh__Array_new__init, KNH_FLAG_MF_VARARGS, CLASS_Array, METHODN_new__init, 0, MFN_nThis_Any1_, (void*)NULL},
-	{knh__Array_add, 0, CLASS_Array, METHODN_add, 0, MFN_void_Any1, (void*)NULL},
-	{knh__Array_opLshift, KNH_FLAG_MF_VARARGS, CLASS_Array, METHODN_opLshift, 0, MFN_void_Any1_, (void*)NULL},
+	{knh__Array_new__array, 0, CLASS_Array, METHODN_new__array, 0, MFN_nThis_nInt, (void*)NULL},
+	{knh__Array_new__init, KNH_FLAG_MF_VARARGS, CLASS_Array, METHODN_new__init, 0, MFN_nThis_nAny1_, (void*)NULL},
+	{knh__IArray_new, 0, CLASS_IArray, METHODN_new, 0, MFN_nThis_Int, (void*)NULL},
+	{knh__IArray_new__array, 0, CLASS_IArray, METHODN_new__array, 0, MFN_nThis_nInt, (void*)NULL},
+	{knh__IArray_new__init, KNH_FLAG_MF_VARARGS, CLASS_IArray, METHODN_new__init, 0, MFN_nThis_nInt_, (void*)NULL},
+	{knh__FArray_new, 0, CLASS_FArray, METHODN_new, 0, MFN_nThis_Int, (void*)NULL},
+	{knh__FArray_new__array, 0, CLASS_FArray, METHODN_new__array, 0, MFN_nThis_nInt, (void*)NULL},
+	{knh__FArray_new__init, KNH_FLAG_MF_VARARGS, CLASS_FArray, METHODN_new__init, 0, MFN_nThis_nFloat_, (void*)NULL},
+	{knh__Array_add, 0, CLASS_Array, METHODN_add, 0, MFN_void_nAny1, (void*)NULL},
+	{knh__Array_opLshift, KNH_FLAG_MF_VARARGS, CLASS_Array, METHODN_opLshift, 0, MFN_void_nAny1_, (void*)NULL},
+	{knh__IArray_add, 0, CLASS_IArray, METHODN_add, 0, MFN_void_nInt, (void*)NULL},
+	{knh__IArray_opLshift, KNH_FLAG_MF_VARARGS, CLASS_IArray, METHODN_opLshift, 0, MFN_void_nInt_, (void*)NULL},
+	{knh__FArray_add, 0, CLASS_FArray, METHODN_add, 0, MFN_void_nFloat, (void*)NULL},
+	{knh__FArray_opLshift, KNH_FLAG_MF_VARARGS, CLASS_FArray, METHODN_opLshift, 0, MFN_void_nFloat_, (void*)NULL},
 	{knh__Array_opSize, 0, CLASS_Array, METHODN_opSize, 0, MFN_nInt, (void*)NULL},
-	{knh__Array_get, 0, CLASS_Array, METHODN_get, 0, MFN_Any1_nInt, (void*)NULL},
-	{knh__Array_set, 0, CLASS_Array, METHODN_set, 0, MFN_void_nInt_Any1, (void*)NULL},
+	{knh__Array_get, 0, CLASS_Array, METHODN_get, 0, MFN_nAny1_nInt, (void*)NULL},
+	{knh__IArray_get, 0, CLASS_IArray, METHODN_get, 0, MFN_nInt_nInt, (void*)NULL},
+	{knh__FArray_get, 0, CLASS_FArray, METHODN_get, 0, MFN_nFloat_nInt, (void*)NULL},
+	{knh__Array_set, 0, CLASS_Array, METHODN_set, 0, MFN_void_nInt_nAny1, (void*)NULL},
+	{knh__IArray_set, 0, CLASS_IArray, METHODN_set, 0, MFN_void_nInt_nInt, (void*)NULL},
+	{knh__FArray_set, 0, CLASS_FArray, METHODN_set, 0, MFN_void_nInt_nFloat, (void*)NULL},
 	{knh__Array_setAll, 0, CLASS_Array, METHODN_setAll, 0, MFN_void_Any1, (void*)NULL},
 	{knh__Array_opSubset, 0, CLASS_Array, METHODN_opSubset, 0, MFN_nThis_Int_Int, (void*)NULL},
 	{knh__Array_opSubsete, 0, CLASS_Array, METHODN_opSubsete, 0, MFN_nThis_Int_Int, (void*)NULL},
@@ -997,10 +1017,17 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Array_indexOf, 0, CLASS_Array, METHODN_indexOf, 0, MFN_nInt_Any1, (void*)NULL},
 	{knh__Array_lastIndexOf, 0, CLASS_Array, METHODN_lastIndexOf, 0, MFN_nInt_Any1, (void*)NULL},
 	{knh__Array_opHas, 0, CLASS_Array, METHODN_opHas, 0, MFN_nBoolean_Any1, (void*)NULL},
-	{knh__Array_sort, 0, CLASS_Array, METHODN_sort, 0, MFN_void, (void*)NULL},
 	{knh__Array_reverse, 0, CLASS_Array, METHODN_reverse, 0, MFN_void, (void*)NULL},
 	{knh__Array_swap, 0, CLASS_Array, METHODN_swap, 0, MFN_void_nInt_nInt, (void*)NULL},
 	{knh__Array_shuffle, 0, CLASS_Array, METHODN_shuffle, 0, MFN_void, (void*)NULL},
+	{knh__IArray_sort, 0, CLASS_IArray, METHODN_sort, 0, MFN_void, (void*)NULL},
+	{knh__IArray_reverse, 0, CLASS_IArray, METHODN_reverse, 0, MFN_void, (void*)NULL},
+	{knh__IArray_swap, 0, CLASS_IArray, METHODN_swap, 0, MFN_void_nInt_nInt, (void*)NULL},
+	{knh__IArray_shuffle, 0, CLASS_IArray, METHODN_shuffle, 0, MFN_void, (void*)NULL},
+	{knh__FArray_sort, 0, CLASS_FArray, METHODN_sort, 0, MFN_void, (void*)NULL},
+	{knh__FArray_reverse, 0, CLASS_FArray, METHODN_reverse, 0, MFN_void, (void*)NULL},
+	{knh__FArray_swap, 0, CLASS_FArray, METHODN_swap, 0, MFN_void_nInt_nInt, (void*)NULL},
+	{knh__FArray_shuffle, 0, CLASS_FArray, METHODN_shuffle, 0, MFN_void, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_Array, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Array__k},
 	{knh__Array_opItr, 0, CLASS_Array, METHODN_opItr, 0, MFN_iAny1, (void*)NULL},
 	{knh__Array_new__array2D, 0, CLASS_Array, METHODN_new__array2D, 0, MFN_nThis_nInt_nInt, (void*)NULL},
@@ -1009,6 +1036,18 @@ static knh_MethodData_t MethodData[] = {
 	{knh__Array_get3D, 0, CLASS_Array, METHODN_get3D, 0, MFN_Any1_nInt_nInt_nInt, (void*)NULL},
 	{knh__Array_set2D, 0, CLASS_Array, METHODN_set2D, 0, MFN_void_nInt_nInt_Any1, (void*)NULL},
 	{knh__Array_set3D, 0, CLASS_Array, METHODN_set3D, 0, MFN_void_nInt_nInt_nInt_Any1, (void*)NULL},
+	{knh__IArray_new__array2D, 0, CLASS_IArray, METHODN_new__array2D, 0, MFN_nThis_nInt_nInt, (void*)NULL},
+	{knh__IArray_new__array3D, 0, CLASS_IArray, METHODN_new__array3D, 0, MFN_nThis_nInt_nInt_nInt, (void*)NULL},
+	{knh__IArray_get2D, 0, CLASS_IArray, METHODN_get2D, 0, MFN_nInt_nInt_nInt, (void*)NULL},
+	{knh__IArray_get3D, 0, CLASS_IArray, METHODN_get3D, 0, MFN_nInt_nInt_nInt_nInt, (void*)NULL},
+	{knh__IArray_set2D, 0, CLASS_IArray, METHODN_set2D, 0, MFN_void_nInt_nInt_nInt, (void*)NULL},
+	{knh__IArray_set3D, 0, CLASS_IArray, METHODN_set3D, 0, MFN_void_nInt_nInt_nInt_nInt, (void*)NULL},
+	{knh__FArray_new__array2D, 0, CLASS_FArray, METHODN_new__array2D, 0, MFN_nThis_nInt_nInt, (void*)NULL},
+	{knh__FArray_new__array3D, 0, CLASS_FArray, METHODN_new__array3D, 0, MFN_nThis_nInt_nInt_nInt, (void*)NULL},
+	{knh__FArray_get2D, 0, CLASS_FArray, METHODN_get2D, 0, MFN_nFloat_nInt_nInt, (void*)NULL},
+	{knh__FArray_get3D, 0, CLASS_FArray, METHODN_get3D, 0, MFN_nFloat_nInt_nInt_nInt, (void*)NULL},
+	{knh__FArray_set2D, 0, CLASS_FArray, METHODN_set2D, 0, MFN_void_nInt_nInt_nFloat, (void*)NULL},
+	{knh__FArray_set3D, 0, CLASS_FArray, METHODN_set3D, 0, MFN_void_nInt_nInt_nInt_nFloat, (void*)NULL},
 	{knh__Bytes_new, 0, CLASS_Bytes, METHODN_new, 0, MFN_nBytes_Int, (void*)NULL},
 	{knh__Bytes_putc, 0, CLASS_Bytes, METHODN_putc, 0, MFN_void_nInt, (void*)NULL},
 	{knh__Bytes_write, 0, CLASS_Bytes, METHODN_write, 0, MFN_void_nBytes_Int_Int, (void*)NULL},
@@ -1047,9 +1086,9 @@ static knh_MethodData_t MethodData[] = {
 	{knh_fmethod_movableText, 0, CLASS_Exception, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_Exception__k},
 	{knh_fmethod_movableText, 0, CLASS_Exception, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Exception__dump},
 	{knh__HashMap_new, 0, CLASS_HashMap, METHODN_new, 0, MFN_nThis_Int, (void*)NULL},
-	{knh__HashMap_get, 0, CLASS_HashMap, METHODN_get, 0, MFN_void_nAny1, (void*)NULL},
-	{knh__HashMap_opHas, 0, CLASS_HashMap, METHODN_opHas, 0, MFN_nBoolean_nAny1, (void*)NULL},
-	{knh__HashMap_remove, 0, CLASS_HashMap, METHODN_remove, 0, MFN_void_nAny1, (void*)NULL},
+	{knh__HashMap_get, 0, CLASS_HashMap, METHODN_get, 0, MFN_void_Any1, (void*)NULL},
+	{knh__HashMap_opHas, 0, CLASS_HashMap, METHODN_opHas, 0, MFN_nBoolean_Any1, (void*)NULL},
+	{knh__HashMap_remove, 0, CLASS_HashMap, METHODN_remove, 0, MFN_void_Any1, (void*)NULL},
 	{knh__HashMap_set, 0, CLASS_HashMap, METHODN_set, 0, MFN_void_nAny1_Any2, (void*)NULL},
 	{knh_fmethod_movableText, 0, CLASS_HashMap, METHODN__k, 0, MFN_void_OutputStream_Any, (void*)knh_HashMap__k},
 	{knh_fmethod_movableText, 0, CLASS_HashMap, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_HashMap__dump},
@@ -1233,6 +1272,9 @@ static knh_MethodData_t MethodData[] = {
 	{knh_fmethod_movableText, 0, CLASS_Script, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Script__dump},
 	{knh__System_getProperty, 0, CLASS_System, METHODN_getProperty, 0, MFN_Any_nString, (void*)NULL},
 	{knh__System_setProperty, 0, CLASS_System, METHODN_setProperty, 0, MFN_void_nString_Any, (void*)NULL},
+	{knh_fmethod_movableText, 0, CLASS_Int, METHODN__const, 0, MFN_void_OutputStream_Any, (void*)knh_Int__const},
+	{knh_fmethod_movableText, 0, CLASS_Method, METHODN__const, 0, MFN_void_OutputStream_Any, (void*)knh_Method__const},
+	{knh_fmethod_movableText, 0, CLASS_Method, METHODN__c, 0, MFN_void_OutputStream_Any, (void*)knh_Method__c},
 	{knh_fmethod_movableText, 0, CLASS_Stmt, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Stmt__s},
 	{knh_fmethod_movableText, 0, CLASS_Stmt, METHODN__dump, 0, MFN_void_OutputStream_Any, (void*)knh_Stmt__dump},
 	{knh_fmethod_movableText, 0, CLASS_Token, METHODN__s, 0, MFN_void_OutputStream_Any, (void*)knh_Token__s},
