@@ -39,7 +39,7 @@
 #else
 #define KONOHA_NAME     "konoha"
 #define KONOHA_MAJOR_VERSION "0.3"
-#define KONOHA_VERSION  "0.3.2"
+#define KONOHA_VERSION  "0.3.2.1"
 #endif
 
 #define KONOHA_URL		"http://konoha.sourceforge.jp/"
@@ -48,6 +48,34 @@
 #define KONOHA_BUILDID_TOOOLD  360
 #define KNH_CHECK_VERSION(id)  ((id) <= KONOHA_BUILDID_TOOOLD)
 
+
+/* ======================================================================== */
+/* COMMON */
+
+#define KONOHA_TSTRUCT_SIZE                64
+#define KONOHA_TCLASS_SIZE                 256
+#define KONOHA_TEXP_SIZE                   128
+
+/* object specification */
+#define KONOHA_PAGESIZE                    4096
+#define KONOHA_FIELDSIZE                   64
+#define KONOHA_STACKSIZE                   4096
+
+#define KONOHA_SMALLPAGESIZE               256
+
+
+#define KONOHA_ERROR_MSG_SIZE              256
+#define KONOHA_INT_SETBUF                  256
+#define KONOHA_DEFAULT_ARRAY_SIZE           16
+#define KONOHA_STRING_BUFSIZ               128
+#define CLASSNAME_BUFSIZ                   128
+#define FILENAME_BUFSIZ                    512
+
+/* object specification */
+
+#define KNH_USING_I18N                     1
+#define KONOHA_ENCODING                    "UTF-8"
+
 /* ======================================================================== */
 /* [OPTIONS] */
 
@@ -172,29 +200,6 @@
 //#define KNH_USING_WORDKLRC   1
 #endif
 
-/* ======================================================================== */
-/* COMMON */
-
-#define KONOHA_TSTRUCT_SIZE                64
-//#define KONOHA_TGLOBAL_SIZE              256
-#define KONOHA_TCLASS_SIZE                 256
-#define KONOHA_TEXP_SIZE                   128
-
-/* object specification */
-
-#define KONOHA_FIELDSIZE                   64
-#define KNH_STACKSIZE                      4096
-#define KONOHA_ERROR_MSG_SIZE              256
-#define KONOHA_INT_SETBUF                  256
-#define KONOHA_DEFAULT_ARRAY_SIZE           16
-#define KONOHA_STRING_BUFSIZ               128
-#define CLASSNAME_BUFSIZ                   128
-#define FILENAME_BUFSIZ                    512
-
-/* object specification */
-
-#define KNH_USING_I18N                     1
-#define KONOHA_ENCODING                    "UTF-8"
 
 /* ======================================================================== */
 /* Language Specification */
