@@ -679,10 +679,19 @@ typedef struct knh_ContextShare_t {
 
 /* ------------------------------------------------------------------------ */
 
+typedef struct knh_ObjectTable_t {
+	int    ctxid;
+	knh_Object_t *table;
+} knh_ObjectTable_t ;
+
+/* ------------------------------------------------------------------------ */
+
 typedef struct knh_Context_t {
 	knh_hObject_t h;
 	knh_Object_t        *unusedObject;
 	size_t               unusedObjectSize;
+	knh_Object_t        *recentCreatedObject;
+	size_t               recentCreatedObjectSize;
 
 	/* stack */
 	knh_sfp_t*                   stack;
@@ -694,7 +703,7 @@ typedef struct knh_Context_t {
 	struct knh_ContextShare_t   *share;
 
 	/* system table */
-	knh_Object_t **tObjectTable;
+	knh_ObjectTable_t *tObjectTable;
 	knh_tStruct_t  *tStruct;
 	knh_tClass_t   *tClass;
 	knh_tExpt_t    *tExpt;
