@@ -269,24 +269,24 @@ int knh_Method_pctoline(Method *mtd, knh_code_t *pc);
 		((Context*)ctx)->esp = &(sfp[n]); \
 	} \
 
-#define KLR_ACALL(ctx, n, shift, mn, reqt) { \
+#define KLR_ACALL(ctx, n, shift, mn) { \
 		Method *mtd_ = knh_Method_ufind(ctx, knh_Object_cid(sfp[n+1].o), mn);\
 		KLR_MOV(ctx, sfp[n].o, mtd_); \
 		((Context*)ctx)->esp = &(sfp[n + shift]); \
-		knh_sfp_typecheck(ctx, sfp + n + 1, reqt, mtd_, pc); \
+		knh_sfp_typecheck(ctx, sfp + n + 1, mtd_, pc); \
 		sfp[-1].pc = pc; \
 		sfp[n].pc = (sfp[n].mtd)->pc_start; \
 		(mtd_)->fcall_1(ctx, sfp + n + 1); \
 		((Context*)ctx)->esp = &(sfp[n]); \
-		TODO(/* AMAP */); \
 	} \
 
 /* this is used only for Closure.invoke */
 
-#define KLR_ASCALL(ctx, n, shift, reqt, m) { \
+#define KLR_AINVOKE(ctx, n, shift, m) { \
 		KLR_MOV(ctx, sfp[n].o, m); \
 		((Context*)ctx)->esp = &(sfp[n + shift]); \
-		knh_sfp_typecheck(ctx, sfp + n + 1, reqt, m, pc); \
+		DBG2_ASSERT(IS_Closure(sfp[n+1].o));\
+		knh_sfp_typecheck(ctx, sfp + n + 1, DP(sfp[n+1].cc)->mtd, pc); \
 		sfp[-1].pc = pc; \
 		sfp[n].pc = (sfp[n].mtd)->pc_start; \
 		(sfp[n].mtd)->fcall_1(ctx, sfp + n + 1); \
