@@ -29,52 +29,155 @@
 
 #include<konoha.h>
 
+#define KNH_PKGNAME "socket"
+
+#ifdef KNH_USING_SOCKET
+#include <sys/socket.h>
+#include <sys/types.h>
+#include <netinet/in.h>
+#include <netdb.h>
+#include <arpa/inet.h>
+#include <unistd.h>
+#endif
+
+
 #ifdef __cplusplus
 extern "C" {
 #endif
 
+/* ======================================================================== */
+/* [SOCKET] */
+
+#ifdef KNH_USING_SOCKET
+
+static
+knh_io_t knh_socket_open(Ctx *ctx, char *ip_or_host, int port)
+{
+	struct in_addr addr = {0};
+	struct hostent	*host;
+	struct sockaddr_in	server = {0};
+	int sd = -1;
+
+	server.sin_family = AF_INET;
+
+	if ((addr.s_addr = inet_addr(ip_or_host)) == -1) {
+		host = gethostbyname(ip_or_host);
+		if (host == NULL) {
+			KNH_PERRNO(ctx, "Socket!!", "gethostbyname");
+			//KNH_THROWf(ctx, "Socket!!: no such host %s", ip_or_host);
+			return -1;
+		}
+		memcpy(&addr, (struct in_addr *)*host->h_addr_list, sizeof(struct in_addr));
+	}
+	server.sin_family = AF_INET;
+	server.sin_addr = addr;
+	server.sin_port = htons(port);
+
+	if((sd = socket(AF_INET, SOCK_STREAM, 0)) < 0) {
+		KNH_PERRNO(ctx, "Socket!!", "socket");
+		return -1;
+	}
+	if (connect(sd, (struct sockaddr *)&server, sizeof(server)) == -1) {
+		KNH_PERRNO(ctx, "Socket!!", "connect");
+		return -1;
+	}
+	return (knh_io_t)sd;
+}
+
+/* ------------------------------------------------------------------------ */
+/* 'ip:127.0.0.1:80' */
+/* 'host:localhost:80' */
+/* 'socket:localhost:80' */
+
+static
+knh_io_t knh_iodrv_open__SOCKET(Ctx *ctx, InputStream *in, OutputStream *out, knh_bytes_t file, char *mode)
+{
+	knh_bytes_t urn = knh_bytes_skipscheme(file);
+	knh_index_t loc = knh_bytes_rindex(urn, ':');
+	int port = 80; /* default */
+	if(loc != -1) {
+		port = (int)knh_bytes_toint(knh_bytes_last(urn, loc+1));
+		urn = knh_bytes_first(urn, loc);
+	}
+	char host_or_ip[128];
+	knh_format_bytes(host_or_ip, sizeof(host_or_ip), urn);
+
+	KNH_WARNING(ctx, "opening socket host='%s', port=%d", host_or_ip, port);
+	KNH_SECURE(ctx);
+
+	knh_io_t sd = knh_socket_open(ctx, host_or_ip, port);
+	if(sd != -1) {
+
+	}
+	return sd;
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+knh_int_t knh_iodrv_read__SOCKET(Ctx *ctx, knh_io_t sd, char *buf, size_t bufsiz)
+{
+	ssize_t ssize = recv((int)sd, buf, bufsiz, 0);
+	return ssize;
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+knh_int_t knh_iodrv_write__SOCKET(Ctx *ctx, knh_io_t sd, char *buf, size_t bufsiz)
+{
+	ssize_t ssize = send((int)sd, buf, bufsiz, 0);
+	return ssize;
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_iodrv_close__SOCKET(Ctx *ctx, knh_io_t sd)
+{
+	close((int)sd);
+}
+
+/* ------------------------------------------------------------------------ */
+/* @data */
+
+static knh_iodrv_t IO__SOCKET = {
+	KNH_DRVAPI_TYPE__IO, "socket",
+	1024,
+	knh_iodrv_open__SOCKET,
+	knh_iodrv_read__SOCKET,
+	knh_iodrv_write__SOCKET,
+	knh_iodrv_close__SOCKET
+};
+
+#endif/*KNH_USING_SOCKET*/
+
 ///* ======================================================================== */
-///* deps/api/knh_socket__{posix|win32|btron}.c */
+///* [structs] */
 //
-//int  knh_socket_open(Ctx *ctx, char *ip_or_host, int port);
-//void knh_socket_close(Ctx *ctx, int sd);
+//typedef struct {
+//	int sd;
+//	int port;
+//	struct knh_InputStream_t  *in;
+//	struct knh_OutputStream_t *out;
+//} knh_Socket_struct ;
 //
-//knh_bool_t knh_Socket_open__deps(Ctx *ctx, knh_Socket *b, char *ip_or_host, int port);
-//void knh_Socket_close__deps(Ctx *ctx, knh_Socket *b);
-//int knh_InputStream_getc__Socket(Ctx *ctx, knh_inptr_t *ptr);
-//size_t knh_InputStream_read__Socket(Ctx *ctx, knh_inptr_t *ptr, char *buf, size_t bufsiz);
-//void knh_OutputStream_putc__Socket(Ctx *ctx, knh_outptr_t *ptr, int ch);
-//size_t knh_OutputStream_write__Socket(Ctx *ctx, knh_outptr_t *ptr, char *buf, size_t bufsiz);
-//void knh_OutputStream_flush__Socket(Ctx *ctx, knh_outptr_t *ptr);
-//
-//static knh_inptr_apis_t DRIVERS__sockin = {
-//	knh_InputStream_open__nop,
-//	knh_InputStream_getc__Socket,
-//	knh_InputStream_read__Socket,
-//	knh_InputStream_close__nop
-//};
-//
-//static knh_outptr_apis_t DRIVERS__sockout = {
-//	knh_OutputStream_open__nop,
-//	knh_OutputStream_putc__Socket,
-//	knh_OutputStream_write__Socket,
-//	knh_OutputStream_flush__Socket,
-//	knh_OutputStream_close__nop
-//};
+//typedef struct {
+//	knh_hObject_t h;
+//	knh_Socket_struct *b;
+//} knh_Socket_t;
 //
 ///* ======================================================================== */
 ///* [structs] */
 //
-//void knh_Socket_struct_init(Ctx *ctx, Struct *s, int init, Object *cs)
+//void knh_Socket_init(Ctx *ctx, Struct *s, int init)
 //{
-//	knh_Socket* b = (knh_Socket*)s;
+//	knh_Socket_struct *b = (knh_Socket_struct*)s;
 //	KNH_INITv(b->urn, knh_String_EMPTY());
 //	b->sd = -1;
 //	b->port = 0;
-//	b->fp_in = NULL;
-//	b->fp_out = NULL;
-//	KNH_INITv(b->in, KNH_NULL);
-//	KNH_INITv(b->out, KNH_NULL);
+//	KNH_INITv(b->in, knh_tClass_defaultValue(ctx, CLASS_InputStream));
+//	KNH_INITv(b->out, knh_tClass_defaultValue(ctx, CLASS_OutputStream));
 //}
 //
 ///* ------------------------------------------------------------------------ */
@@ -88,7 +191,7 @@ extern "C" {
 ///* ------------------------------------------------------------------------ */
 //
 //void
-//knh_Socket_struct_traverse(Ctx *ctx, Struct *s, f_traverse gc)
+//knh_Socket_traverse(Ctx *ctx, Struct *s, f_traverse gc)
 //{
 //	knh_Socket *b = (knh_Socket*)s;
 //	gc(ctx, b->urn);
@@ -108,7 +211,7 @@ extern "C" {
 //{
 //	char *ip_or_host = NULL;
 //	Socket* b = (Socket*)knh_Object_malloc(ctx, CLASS_Socket);
-//	knh_Socket_struct_init(ctx, (Struct*)b, 0, NULL);
+//	knh_Socket_init(ctx, (Struct*)b, 0, NULL);
 //	KNH_SETv(ctx, b->urn, new_StringX__fast(ctx, CLASS_String, urn));
 //	b->port = port;
 //
@@ -163,27 +266,52 @@ extern "C" {
 //		b->sd = -1;
 //	}
 //}
-//
+
 
 /* ======================================================================== */
 /* [KNHAPI] */
 
-KNHAPI(int) knhsocket_init(Ctx *ctx)
+int knhsocket_init(Ctx *ctx)
 {
 	KNH_NOTICE(ctx, "loading socket..");
-
+#ifdef KNH_USING_SOCKET
+	konoha_addIODriver(ctx, NULL, &IO__SOCKET);
+	konoha_addIODriver(ctx, "ip", &IO__SOCKET);
+	konoha_addIODriver(ctx, "host", &IO__SOCKET);
+#endif
 	return 1;
 }
 
 /* ------------------------------------------------------------------------ */
 
-KNHAPI(void) knhsocket_traverse(Ctx *ctx, f_traverse ftr)
+void knhsocket_traverse(Ctx *ctx, knh_ftraverse ftr)
 {
-	if(konoha_issweep(ftr)) {
+	if(IS_SWEEP(ftr)) {
 
 	}
 }
 
+
+/* ------------------------------------------------------------------------ */
+
+static knh_pkgmeta_t PKGDATA = {
+	KONOHA_BUILDID, /* Don't change this */
+	KNH_PKGNAME,    /* package name (not NULL) */
+	"1.0",  /* package version (not NULL) */
+	NULL,   /* additional information */
+	KONOHA_URLBASE "package.socket", /* URL */
+	knhsocket_init,
+	knhsocket_traverse,
+	NULL
+};
+
+KNHAPI(knh_pkgmeta_t*) knhsocket_load(Ctx *ctx)
+{
+	return &(PKGDATA);
+}
+
+/* ------------------------------------------------------------------------ */
+
 /* ------------------------------------------------------------------------ */
 
 #ifdef __cplusplus
