@@ -36,7 +36,29 @@ extern "C" {
 #endif
 
 /* ======================================================================== */
-/* [constructor] */
+/* [common] */
+
+static
+knh_hcode_t knh_stack_hashCode(Ctx *ctx, knh_sfp_t *sfp)
+{
+	knh_class_t bcid = (sfp[0].o)->h.bcid;
+	if(CLASS_Boolean <= bcid && bcid <= CLASS_Float) {
+		return (knh_hcode_t)(sfp[0].i)->n.data;
+	}
+	return ctx->tStruct[bcid].fhashCode(ctx, sfp[0].o);
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+int knh_stack_equals(Ctx *ctx, knh_sfp_t *sfp, Object *o)
+{
+	knh_class_t bcid = (sfp[0].o)->h.bcid;
+	if(CLASS_Boolean <= bcid && bcid <= CLASS_Float) {
+		return ((sfp[0].i)->n.data == ((Int*)o)->n.data);
+	}
+	return (knh_Object_compareTo(ctx, sfp[0].o, o) == 0);
+}
 
 /* ------------------------------------------------------------------------ */
 /* @method This! HashMap.new(Int init) */
@@ -50,90 +72,73 @@ METHOD knh__HashMap_new(Ctx *ctx, knh_sfp_t *sfp)
 	}
 	KNH_RETURN(ctx, sfp, o);
 }
-/* ------------------------------------------------------------------------ */
-/* ======================================================================== */
-/* [method] */
 
 /* ------------------------------------------------------------------------ */
-/* @method void HashMap.get(Any1! key) */
+/* @method void HashMap.get(Any1 key) */
 
 METHOD knh__HashMap_get(Ctx *ctx, knh_sfp_t *sfp)
 {
 	HashMap *o = (HashMap*)sfp[0].o;
-	knh_sfp_boxing(ctx, sfp+1);
-	Object *key = sfp[1].o;
+	knh_hcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
+	knh_uint_t h =  hcode % DP(o)->hmax;
+	knh_hashentry_t *e = DP(o)->array[h];
 
-	knh_hcode_t hcode = knh_Object_hashCode(ctx, key);
-	knh_uint_t h =  hcode % DP(o)->capacity;
-	knh_hashmapentry_t *cur = DP(o)->array[h];
-
-	while(cur != NULL) {
-		if(cur->hcode == hcode) {
-			if(DP(o)->fcompareTo(ctx, cur->key, key) == 0 || IS_NULL(cur->key)) {
-				KNH_RETURN(ctx, sfp, cur->value);
-			}
+	while(e != NULL) {
+		if(e->hcode == hcode
+				&& knh_Object_cid(sfp[1].o) == knh_Object_cid(e->key)
+				&& knh_stack_equals(ctx, sfp + 1, e->key)) {
+			KNH_RETURN(ctx, sfp, e->value);
 		}
-		cur = cur->next;
+		e = e->next;
 	}
 	KNH_RETURN(ctx, sfp, KNH_NULL);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method Boolean! HashMap.opHas(Any1! key) */
+/* @method Boolean! HashMap.opHas(Any1 key) */
 
 METHOD knh__HashMap_opHas(Ctx *ctx, knh_sfp_t *sfp)
 {
 	HashMap *o = (HashMap*)sfp[0].o;
-	knh_sfp_boxing(ctx, sfp+1);
-	Object *key = sfp[1].o;
+	knh_hcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
+	knh_uint_t h =  hcode % DP(o)->hmax;
+	knh_hashentry_t *e = DP(o)->array[h];
 
-	knh_hcode_t hcode = knh_Object_hashCode(ctx, key);
-	knh_uint_t h =  hcode % DP(o)->capacity;
-	knh_hashmapentry_t *cur = DP(o)->array[h];
-
-	while(cur != NULL) {
-		if(cur->hcode == hcode) {
-			if(DP(o)->fcompareTo(ctx, cur->key, key) == 0 || IS_NULL(cur->key)) {
-				KNH_RETURN_Boolean(ctx, sfp, 1);
-			}
+	while(e != NULL) {
+		if(e->hcode == hcode
+				&& knh_Object_cid(sfp[1].o) == knh_Object_cid(e->key)
+				&& knh_stack_equals(ctx, sfp + 1, e->key)) {
+			KNH_RETURN_Boolean(ctx, sfp, 1);
 		}
-		cur = cur->next;
+		e = e->next;
 	}
 	KNH_RETURN_Boolean(ctx, sfp, 0);
 }
 
 /* ------------------------------------------------------------------------ */
-/* @method void HashMap.remove(Any1! key) */
+/* @method void HashMap.remove(Any1 key) */
 
 METHOD knh__HashMap_remove(Ctx *ctx, knh_sfp_t *sfp)
 {
-	HashMap *o = (HashMap*)sfp[0].o;
-	if(!knh_Object_isImmutable(o)) {
-		knh_sfp_boxing(ctx, sfp+1);
-		Object *key = sfp[1].o;
+	KNH_CHECK_IMMUTABLE(ctx, sfp[0].o, L_RETURN);
+	knh_Hash_t *o = (knh_Hash_t*)sfp[0].o;
+	knh_hcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
+	knh_uint_t h =  hcode % DP(o)->hmax;
+	knh_hashentry_t *e = DP(o)->array[h];
+	knh_hashentry_t **prev_next = &(DP(o)->array[h]);
 
-		knh_hcode_t hcode = knh_Object_hashCode(ctx, key);
-		knh_uint_t h =  hcode % DP(o)->capacity;
-		knh_hashmapentry_t *cur = DP(o)->array[h];
-		knh_hashmapentry_t **prev_next = &(DP(o)->array[h]);
-
-		while(cur != NULL) {
-			if(cur->hcode == hcode) {
-				if(DP(o)->fcompareTo(ctx, cur->key, key) == 0 || IS_NULL(cur->key)) {
-					prev_next[0] = cur->next;
-					cur->next = DP(o)->unused;
-					DP(o)->unused = cur;
-					cur->hcode = 0;
-					cur->key = NULL;
-					cur->value = NULL;
-					DP(o)->size--;
-					break;
-				}
-			}
-			prev_next = &(cur->next);
-			cur = cur->next;
+	while(e != NULL) {
+		if(e->hcode == hcode
+				&& knh_Object_cid(sfp[1].o) == knh_Object_cid(e->key)
+				&& knh_stack_equals(ctx, sfp + 1, e->key)) {
+			prev_next[0] = e->next;
+			knh_hashentry_collect(ctx, o, e);
+			DP(o)->size--;
 		}
+		prev_next = &(e->next);
+		e = e->next;
 	}
+	L_RETURN:;
 	KNH_RETURN_void(ctx, sfp);
 }
 
@@ -142,50 +147,53 @@ METHOD knh__HashMap_remove(Ctx *ctx, knh_sfp_t *sfp)
 
 METHOD knh__HashMap_set(Ctx *ctx, knh_sfp_t *sfp)
 {
+	KNH_CHECK_IMMUTABLE(ctx, sfp[0].o, L_RETURN);
 	if(IS_NULL(sfp[2].o)) {
 		knh__HashMap_remove(ctx, sfp);
-	}else if(!knh_Object_isImmutable(sfp[0].o)) {
-		HashMap *o = (HashMap*)sfp[0].o;
-		knh_sfp_boxing(ctx, sfp+1);
-		Object *key = sfp[1].o;
-		knh_sfp_boxing(ctx, sfp+2);
-		Object *value  = sfp[2].o;
+		return ;
+	}
 
-		knh_hcode_t hcode = knh_Object_hashCode(ctx, key);
-		knh_uint_t h =  hcode % DP(o)->capacity;
-		knh_hashmapentry_t *cur = DP(o)->array[h];
-		while(cur != NULL) {
-			if(cur->hcode == hcode) {
-				if(DP(o)->fcompareTo(ctx, cur->key, key) == 0 || IS_NULL(cur->key)) {
-					KNH_SETv(ctx, cur->value, value);
-					KNH_RETURN_void(ctx, sfp);
-				}
-			}
-			cur = cur->next;
+	knh_Hash_t *o = (knh_Hash_t*)sfp[0].o;
+	knh_hcode_t hcode = knh_stack_hashCode(ctx, sfp + 1);
+	knh_uint_t h =  hcode % DP(o)->hmax;
+	knh_hashentry_t *e = DP(o)->array[h];
+
+	while(e != NULL) {
+		if(e->hcode == hcode
+				&& knh_Object_cid(sfp[1].o) == knh_Object_cid(e->key)
+				&& knh_stack_equals(ctx, sfp + 1, e->key)) {
+			knh_sfp_boxing(ctx, sfp+2);
+			KNH_SETv(ctx, e->value, sfp[2].o);
+			goto L_RETURN;
 		}
-		cur = new_hashmapentry(ctx, o);
-		cur->hcode = hcode;
-		KNH_INITv(cur->key, key);
-		KNH_INITv(cur->value, value);
-		cur->next = DP(o)->array[h];
-		DP(o)->array[h] = cur;
+		e = e->next;
+	}
+
+	/* add newentry */ {
+		e = new_hashentry(ctx, o);
+		e->hcode = hcode;
+		knh_sfp_boxing(ctx, sfp+1);
+		KNH_INITv(e->key, sfp[1].o);
+		knh_sfp_boxing(ctx, sfp+2);
+		KNH_INITv(e->value, sfp[2].o);
+		e->next = DP(o)->array[h];
+		DP(o)->array[h] = e;
 		DP(o)->size++;
 	}
+	L_RETURN:;
 	KNH_RETURN_void(ctx, sfp);
 }
 
 /* ======================================================================== */
-/* [movabletext] */
-
-/* ------------------------------------------------------------------------ */
 /* @method void HashMap.%k(OutputStream w, String m) */
 
-void knh_HashMap__k(Ctx *ctx, HashMap *o, OutputStream *w, String *m)
+void knh_HashMap__k(Ctx *ctx, knh_Hash_t *o, OutputStream *w, String *m)
 {
+	size_t pos = 0, c = 0;
+	size_t max = (KNH_HASH_TABLESIZE / o->hashop->size) * DP(o)->tables_size;
 	knh_putc(ctx, w, '{');
-	size_t pos = 0, capacity = knh_hashmapentry_capacity(ctx, o), c = 0;
-	while(pos < capacity) {
-		knh_hashmapentry_t *e = knh_hashmapentry_at(o, pos);
+	while(pos < max) {
+		knh_hashentry_t *e = knh_hashentry_at((knh_Hash_t*)o, pos);
 		if(e != NULL) {
 			if(c > 0) {
 				knh_write_delim(ctx,w);
@@ -206,7 +214,7 @@ void knh_HashMap__k(Ctx *ctx, HashMap *o, OutputStream *w, String *m)
 
 void knh_HashMap__dump(Ctx *ctx, HashMap *o, OutputStream *w, String *m)
 {
-	knh_HashMap__k(ctx, o, w, m);
+	knh_HashMap__k(ctx, (knh_Hash_t*)o, w, m);
 }
 
 /* ======================================================================== */
@@ -216,11 +224,12 @@ static
 ITRNEXT knh_HashMap_var_next(Ctx *ctx, knh_sfp_t *sfp, int n)
 {
 	Iterator *it = sfp[0].it;
-	HashMap *o = (HashMap*)DP(it)->source;
+	knh_Hash_t *o = (knh_Hash_t*)DP(it)->source;
 	KNH_ASSERT(IS_bHashMap(o));
-	size_t pos = DP(it)->pos, capacity = knh_hashmapentry_capacity(ctx, o);
-	while(pos < capacity) {
-		knh_hashmapentry_t *e = knh_hashmapentry_at(o, pos);
+	size_t pos = DP(it)->pos;
+	size_t max = (KNH_HASH_TABLESIZE / o->hashop->size) * DP(o)->tables_size;
+	while(pos < max) {
+		knh_hashentry_t *e = knh_hashentry_at(o, pos);
 		if(e != NULL) {
 			DP(it)->pos = pos + 1;
 			KNH_ITRNEXT(ctx, sfp, n, e->key);
