@@ -151,7 +151,7 @@ _SS_ METHOD knh__Int_opGte(Ctx *ctx, knh_sfp_t *sfp)
 
 _SS_ METHOD knh__Int_opLand(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_vargc_t ac = knh_sfp_argc(ctx, sfp + 1);
+	int ac = knh_sfp_argc(ctx, sfp + 1);
 	knh_sfp_t *v = sfp + 1;
 	knh_integer_t n = p_integer(sfp[0]);
 	size_t i;
@@ -166,7 +166,7 @@ _SS_ METHOD knh__Int_opLand(Ctx *ctx, knh_sfp_t *sfp)
 
 _SS_ METHOD knh__Int_opLor(Ctx *ctx, knh_sfp_t *sfp)
 {
-	knh_vargc_t ac = knh_sfp_argc(ctx, sfp + 1);
+	int ac = knh_sfp_argc(ctx, sfp + 1);
 	knh_sfp_t *v = sfp + 1;
 	knh_integer_t n = p_integer(sfp[0]);
 	size_t i;
@@ -570,7 +570,7 @@ _SS_ METHOD knh__Float__bits(Ctx *ctx, knh_sfp_t *sfp)
 _SS_ METHOD knh__Float__k(Ctx *ctx, knh_sfp_t *sfp)
 {
 	Float *o = sfp[0].f;
-	FloatUnit *u = (FloatUnit*)knh_tClass[o->h.cid].cspec;
+	FloatUnit *u = (FloatUnit*)ctx->tClass[o->h.cid].cspec;
 	KNH_ASSERT(IS_FloatUnit(u));
 	knh_write__ffmt(ctx, sfp[1].w, DP(u)->FMT, p_float(sfp[0]));
 	knh_bytes_t tag = knh_String_tobytes(DP(u)->spec.tag);
