@@ -187,7 +187,7 @@ KNHAPI(void) knh_ResultSet_initData(Ctx *ctx, ResultSet *o)
 		DP(o)->column[i].start = 0;
 		DP(o)->column[i].len = 0;
 	}
-	knh_Bytes_clear(DP(o)->databuf);
+	knh_Bytes_clear(DP(o)->databuf, 0);
 }
 
 /* ------------------------------------------------------------------------ */
