@@ -607,9 +607,10 @@ void knh_IArray_init(Ctx *ctx, knh_IArray_t *a, int init)
 	a->ilist = NULL;
 	a->capacity = 0;
 	a->size = 0;
-	if(init > 0) {
-		knh_IArray_grow(ctx, a, init, 0);
-	}
+	KNH_ASSERT(init == 0);
+//	if(init > 0) {
+//		knh_IArray_grow(ctx, a, init, 0);
+//	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -647,9 +648,10 @@ void knh_FArray_init(Ctx *ctx, knh_FArray_t *a, int init)
 	a->flist = NULL;
 	a->capacity = 0;
 	a->size = 0;
-	if(init > 0) {
-		knh_FArray_grow(ctx, a, init, 0.0);
-	}
+	KNH_ASSERT(init == 0);
+//	if(init > 0) {
+//		knh_FArray_grow(ctx, a, init, 0.0);
+//	}
 }
 
 /* ------------------------------------------------------------------------ */
@@ -770,7 +772,7 @@ void knh_DictSet_init(Ctx *ctx, knh_DictSet_t *d, int init)
 
 #define knh_HashMap_init_ NULL
 #define knh_HashMap_copy NULL
-#define knh_HashMap_traverse_ NULL
+#define knh_HashMap_traverse knh_Hash_traverse
 #define knh_HashMap_compareTo NULL
 #define knh_HashMap_hashCode NULL
 #define knh_HashMap_newClass NULL
@@ -778,83 +780,94 @@ void knh_DictSet_init(Ctx *ctx, knh_DictSet_t *d, int init)
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_HashMap_init(Ctx *ctx, HashMap *hash, int init)
+void knh_Hash_init(Ctx *ctx, knh_Hash_t *hash, int init)
 {
-	knh_HashMap_struct *b = DP(hash);
+	knh_Hash_struct *b = DP(hash);
 	b->DBG_name = "HashMap";
 	if(init > 0) {
-		size_t i;
-		b->table_size = init * 3 / 4 + 10;
-		b->table = (knh_hashmapentry_t*)KNH_MALLOC(ctx, sizeof(knh_hashmapentry_t) * b->table_size);
-		b->ex_tables = NULL;
-		b->ex_size = 0;
-		b->ex_capacity = 0;
-		for(i = 0; i < b->table_size; i++) {
-			b->table[i].hcode = 0;
-			b->table[i].key = NULL;
-			b->table[i].value = NULL;
-			b->table[i].next = (i == b->table_size - 1) ? NULL : &(b->table[i+1]);
-		}
-		b->unused = &(b->table[0]);
-
-		b->capacity = init;
-		b->array = (knh_hashmapentry_t**)KNH_MALLOC(ctx, sizeof(knh_hashmapentry_t*) * init);
-		knh_bzero(b->array, sizeof(knh_hashmapentry_t*) * init);
+		b->hmax = init;
+		b->capacity = knh_bytes_newsize(init);
+		b->array = (knh_hashentry_t**)KNH_MALLOC(ctx, sizeof(knh_hashentry_t*) * b->capacity);
+		knh_bzero(b->array, sizeof(knh_hashentry_t*) * b->capacity);
 	}
 	else {
-		b->table_size = 0;
-		b->table = NULL;
-		b->ex_tables = NULL;
-		b->ex_size = 0;
-		b->ex_capacity = 0;
-		b->unused = NULL;
-
+		b->hmax = 0;
 		b->capacity = 0;
 		b->array = NULL;
 	}
-	b->fcompareTo = knh_Object_compareTo;
+	b->size = 0;
+	b->tables = NULL;
+	b->tables_size = 0;
+	b->tables_capacity = 0;
+	b->tables_first = NULL;
+	b->unused_first = NULL;
+	//
+	hash->fcmp = knh_Object_compareTo;
 }
 
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_HashMap_traverse(Ctx *ctx, HashMap *hash, knh_ftraverse ftr)
+void knh_hashmapentry_traverse(Ctx *ctx, knh_hashentry_t *e, knh_ftraverse ftr)
 {
-	knh_HashMap_struct *b = DP(hash);
-	size_t i;
-	for(i = 0; i < b->table_size; i++) {
-		if(b->table[i].key != NULL) {
-			ftr(ctx, b->table[i].key);
-			ftr(ctx, b->table[i].value);
-		}
-	}
-	if(b->ex_size > 0) {
-		size_t j;
-		for(j = 0; j < b->ex_size; j++) {
-			knh_hashmapentry_t *t = b->ex_tables[j];
-			for(i = 0; i < b->table_size; i++) {
-				if(t[i].key != NULL) {
-					ftr(ctx, t[i].key);
-					ftr(ctx, t[i].value);
-				}
+	DBG2_ASSERT(e->key != NULL);
+	ftr(ctx, e->key);
+	ftr(ctx, e->value);
+}
+
+/* ------------------------------------------------------------------------ */
+/* @data */
+
+static knh_hash_op knh_hashmap_op = {
+	sizeof(knh_hashentry_t),
+	NULL,
+	knh_hashmapentry_traverse,
+};
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_HashMap_init(Ctx *ctx, knh_HashMap_t *o, int init)
+{
+	knh_Hash_t *hash = (knh_Hash_t*)o;
+	knh_Hash_init(ctx, hash, init);
+	DP(hash)->DBG_name = "HashMap";
+	hash->fcmp = knh_Object_compareTo;
+	hash->hashop = &knh_hashmap_op;
+}
+
+/* ------------------------------------------------------------------------ */
+
+static
+void knh_Hash_traverse(Ctx *ctx, knh_Hash_t *hash, knh_ftraverse ftr)
+{
+	knh_Hash_struct *b = DP(hash);
+	size_t i, j, esize = hash->hashop->size, tsize = KNH_HASH_TABLESIZE / esize;
+	for(j = 0; j < b->tables_size; j++) {
+		for(i = 0; i < tsize; i++) {
+			knh_hashentry_t *e = KNH_HASHENTRY(b->tables[j], esize, i);
+			if(e->key != NULL) {
+				hash->hashop->ftraverse(ctx, e, ftr);
 			}
 		}
 	}
 	if(IS_SWEEP(ftr)) {
-		if(b->table != NULL) {
-			KNH_FREE(ctx, b->table, sizeof(knh_hashmapentry_t) * b->table_size);
-			b->table = NULL;
-		}
 		if(b->array != NULL) {
-			KNH_FREE(ctx, b->array, sizeof(knh_hashmapentry_t*) * b->capacity);
+			KNH_FREE(ctx, b->array, sizeof(knh_hashentry_t*) * b->capacity);
 			b->array = NULL;
+			b->size = 0;
+			b->capacity = 0;
 		}
-		if(b->ex_tables != NULL) {
-			for(i = 0; i < b->ex_size; i++) {
-				KNH_FREE(ctx, b->ex_tables[i], sizeof(knh_hashmapentry_t) * b->table_size);
+		if(b->tables_size > 0) {
+			for(i = 0; i < b->tables_size; i++) {
+				KNH_FREE(ctx, b->tables[i], KNH_HASH_TABLESIZE);
 			}
-			KNH_FREE(ctx, b->ex_tables, sizeof(knh_hashmapentry_t*) * b->ex_capacity);
-			b->ex_tables = NULL;
+			if(b->tables_capacity > 1) {
+				KNH_FREE(ctx, b->tables, sizeof(knh_hashentry_t*) * b->capacity);
+			}
+			b->tables = NULL;
+			b->tables_capacity = 0;
+			b->tables_size = 0;
 		}
 	}
 }
@@ -864,150 +877,41 @@ void knh_HashMap_traverse(Ctx *ctx, HashMap *hash, knh_ftraverse ftr)
 
 #define knh_HashSet_init_ NULL
 #define knh_HashSet_copy NULL
-#define knh_HashSet_traverse_ NULL
+#define knh_HashSet_traverse knh_Hash_traverse
 #define knh_HashSet_compareTo NULL
 #define knh_HashSet_hashCode NULL
 #define knh_HashSet_newClass NULL
 
+/* ------------------------------------------------------------------------ */
+
 static
-void knh_HashSet_init(Ctx *ctx, HashSet *hash, int init)
+void knh_hashsetentry_traverse(Ctx *ctx, knh_hashentry_t *e, knh_ftraverse ftr)
 {
-	knh_HashSet_struct *b = DP(hash);
-	b->DBG_name = "HashSet";
-	if(init > 0) {
-		size_t i;
-		b->table_size = init * 3 / 4 + 10;
-		b->table = (knh_hashsetentry_t*)KNH_MALLOC(ctx, sizeof(knh_hashsetentry_t) * b->table_size);
-		b->ex_tables = NULL;
-		b->ex_size = 0;
-		b->ex_capacity = 0;
-		for(i = 0; i < b->table_size; i++) {
-			b->table[i].hcode = 0;
-			b->table[i].key = NULL;
-			b->table[i].value = 0;
-			b->table[i].next = (i == b->table_size - 1) ? NULL : &(b->table[i+1]);
-		}
-		b->unused = &(b->table[0]);
-
-		b->capacity = init;
-		b->array = (knh_hashsetentry_t**)KNH_MALLOC(ctx, sizeof(knh_hashsetentry_t*) * init);
-		knh_bzero(b->array, sizeof(knh_hashsetentry_t*) * init);
-	}
-	else {
-		b->table_size = 0;
-		b->table = NULL;
-		b->ex_tables = NULL;
-		b->ex_size = 0;
-		b->ex_capacity = 0;
-		b->unused = NULL;
-
-		b->capacity = 0;
-		b->array = NULL;
-	}
-	b->fcompareTo = knh_Object_compareTo;
+	DBG2_ASSERT(e->key != NULL);
+	ftr(ctx, e->key);
 }
 
+/* ------------------------------------------------------------------------ */
+/* @data */
+
+static knh_hash_op knh_hashset_op = {
+	sizeof(knh_hashentry_t),
+	NULL,
+	knh_hashsetentry_traverse,
+};
+
 /* ------------------------------------------------------------------------ */
 
 static
-void knh_HashSet_traverse(Ctx *ctx, knh_HashSet_t *hash, knh_ftraverse ftr)
+void knh_HashSet_init(Ctx *ctx, knh_HashSet_t *o, int init)
 {
-	knh_HashSet_struct *b = DP(hash);
-	size_t i;
-	for(i = 0; i < b->table_size; i++) {
-		if(b->table[i].key != NULL) {
-			ftr(ctx, b->table[i].key);
-		}
-	}
-	if(b->ex_size > 0) {
-		size_t j;
-		for(j = 0; j < b->ex_size; j++) {
-			knh_hashsetentry_t *t = b->ex_tables[j];
-			for(i = 0; i < b->table_size; i++) {
-				if(t[i].key != NULL) {
-					ftr(ctx, t[i].key);
-				}
-			}
-		}
-	}
-	if(IS_SWEEP(ftr)) {
-		if(b->table != NULL) {
-			KNH_FREE(ctx, b->table, sizeof(knh_hashsetentry_t) * b->table_size);
-			b->table = NULL;
-		}
-		if(b->array != NULL) {
-			KNH_FREE(ctx, b->array, sizeof(knh_hashsetentry_t*) * b->capacity);
-			b->array = NULL;
-		}
-		if(b->ex_tables != NULL) {
-			for(i = 0; i < b->ex_size; i++) {
-				KNH_FREE(ctx, b->ex_tables[i], sizeof(knh_hashsetentry_t) * b->table_size);
-			}
-			KNH_FREE(ctx, b->ex_tables, sizeof(knh_hashmapentry_t*) * b->ex_capacity);
-			b->ex_tables = NULL;
-		}
-	}
+	knh_Hash_t *hash = (knh_Hash_t*)o;
+	knh_Hash_init(ctx, hash, init);
+	DP(hash)->DBG_name = "HashSet";
+	hash->fcmp = knh_Object_compareTo;
+	hash->hashop = &knh_hashset_op;
 }
 
-/* ======================================================================== */
-/* LRUHashMap */
-
-#define knh_LRUHashMap_init_ NULL
-#define knh_LRUHashMap_copy NULL
-#define knh_LRUHashMap_traverse_ NULL
-#define knh_LRUHashMap_compareTo NULL
-#define knh_LRUHashMap_hashCode NULL
-#define knh_LRUHashMap_newClass NULL
-
-/* ------------------------------------------------------------------------ */
-
-static
-void knh_LRUHashMap_init(Ctx *ctx, LRUHashMap *hm, int init)
-{
-	knh_LRUHashMap_struct *b = DP(hm);
-	int i;
-	KNH_ASSERT(init > 0);
-	b->table_size = init;
-	b->table = (knh_lruentry_t*)KNH_MALLOC(ctx, sizeof(knh_lruentry_t) * init);
-	for(i = 0; i < init; i++) {
-		b->table[i].hcode = 0;
-		KNH_INITv(b->table[i].key, KNH_NULL);
-		KNH_INITv(b->table[i].value, KNH_NULL);
-		b->table[i].value_next = (i == init -1) ? NULL : &(b->table[i+1]);
-		b->table[i].next = NULL;
-		b->table[i].prev = NULL;
-	}
-	b->unused = &(b->table[0]);
-	b->array = (knh_lruentry_t**)KNH_MALLOC(ctx, sizeof(knh_lruentry_t*) * init);
-	knh_bzero(b->array, sizeof(knh_lruentry_t*) * init);
-	b->first = NULL;
-	b->last = NULL;
-	b->dbg_name = "LRUHashMap";
-	b->hitc = 0;
-	b->missc = 0;
-}
-
-/* ------------------------------------------------------------------------ */
-
-static
-void knh_LRUHashMap_traverse(Ctx *ctx, LRUHashMap *hm, knh_ftraverse ftr)
-{
-	knh_LRUHashMap_struct *b = DP(hm);
-	size_t i;
-	for(i = 0; i < b->table_size; i++) {
-		ftr(ctx, b->table[i].key);
-		ftr(ctx, b->table[i].value);
-	}
-	if(IS_SWEEP(ftr)) {
-		KNH_FREE(ctx, b->table, sizeof(knh_lruentry_t) * b->table_size);
-		KNH_FREE(ctx, b->array, sizeof(knh_lruentry_t*) * b->table_size);
-		b->table = NULL;
-		b->array = NULL;
-		DBG_P("%s[%d] hit=%d miss=%d ratio=%f",
-			b->dbg_name, (int)b->table_size,
-			(int)b->hitc, (int)b->missc, 100.0 * b->hitc / (b->hitc + b->missc + 1));
-	}
-}
 
 /* ======================================================================== */
 /* DictIdx */
@@ -2192,6 +2096,9 @@ void knh_Asm_init(Ctx *ctx, Asm *abr, int init)
 	b->line = 0;
 	KNH_INITv(b->elf, new_Bytes(ctx, 4096));
 	KNH_INITv(b->dwarf, new_Bytes(ctx, 1024));
+
+	KNH_INITv(b->constPools, KNH_NULL);
+	KNH_INITv(b->exportsMethods, KNH_NULL);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -2222,6 +2129,9 @@ void knh_Asm_traverse(Ctx *ctx, Asm *abr, knh_ftraverse ftr)
 	ftr(ctx, UP(b->name2labelIdDictSet));
 	ftr(ctx, UP(b->lstacks));
 	ftr(ctx, UP(b->finallyStmt));
+
+	ftr(ctx, UP(b->exportsMethods));
+	ftr(ctx, UP(b->constPools));
 }
 
 /* ======================================================================== */
