@@ -64,6 +64,7 @@ void *knh_malloc(Ctx *ctx, size_t size)
 
 void knh_free(Ctx *ctx, void *block, size_t size)
 {
+	DBG2_ASSERT(!knh_Context_isMarking(ctx));
 #ifdef KNH_STATMODE0
 	ctx->share->statUsedMemorySize -= size;
 #endif
@@ -93,6 +94,7 @@ void *DBG2_malloc(Ctx *ctx, size_t size, char *func)
 
 void DBG2_free(Ctx *ctx, void *p, size_t size, char *func)
 {
+	DBG2_ASSERT(!knh_Context_isMarking(ctx));
 	size_t *block = ((size_t*)p) - 1;
 #ifdef KNH_STATMODE0
 	ctx->share->statUsedMemorySize -= size;
@@ -159,7 +161,8 @@ knh_Object_t *new_UnusedObject(Ctx *ctx)
 		return NULL;
 	}
 	knh_Object_t *t = (knh_Object_t*)KNH_MALLOC(ctx, SIZEOF_TOBJECT);
-	ctx->tObjectTable[ctxshr->tObjectTableSize] = t;
+	ctx->tObjectTable[ctxshr->tObjectTableSize].ctxid = ctx->ctxid;
+	ctx->tObjectTable[ctxshr->tObjectTableSize].table = t;
 	int i;
 	for(i = 0; i < KNH_TOBJECT_SIZE - 1; i++) {
 		t[i].h.magic = 0;
@@ -247,6 +250,7 @@ void *knh_fastmalloc(Ctx *ctx, size_t size)
 
 void knh_fastfree(Ctx *ctx, void *block, size_t size)
 {
+	DBG2_ASSERT(!knh_Context_isMarking(ctx));
 	if(size <= KNH_FASTMALLOC_SIZE) {
 		knh_Object_t *o = (knh_Object_t*)(((knh_int_t*)block)-1);
 		assert(o->h.magic == KNH_FASTMALLOC_MAGIC);
@@ -390,7 +394,6 @@ void knh_Object_free(Ctx *ctx, knh_Object_t *o)
 //		knh_metadata_clear(ctx, o);
 	}
 #endif
-
 	if(o->h.bcid == CLASS_Context) return;
 	KNH_OBJECT_REUSE(ctx, o);
 #ifdef KNH_STATMODE0
@@ -473,30 +476,56 @@ void knh_Object_mark0(Ctx *ctx, Object *o)
 
 /* ------------------------------------------------------------------------ */
 
-void knh_Object_sweep0(Ctx *ctx, Object *o)
+static
+void knh_Object_finalSweep(Ctx *ctx, Object *o)
 {
-	if(!knh_Object_isGCMarked(o)) {
-		knh_Object_traverse(ctx, o, knh_Object_sweep0);
-	}
+	// DO Nothing;
 }
 
 /* ------------------------------------------------------------------------ */
+/* @method[STATIC] void System.gc() */
 
-void knh_Object_mark1(Ctx *ctx, Object *o)
+METHOD knh__System_gc(Ctx *ctx, knh_sfp_t *sfp)
 {
-	if(!knh_Object_isGCMarked(o)) {
-		knh_Object_setGCMarked(o, 1);
-		knh_Object_traverse(ctx, o, knh_Object_mark1);
+	int i, j, size = (int)(ctx->share->tObjectTableSize);
+	DBG2_P("**** starting gc ****");
+	for(i = 0; i < size; i++) {
+		Object *t = ctx->tObjectTable[i].table;
+		for(j = 0; j < KNH_TOBJECT_SIZE; j++) {
+			Object *o = &t[j];
+			if(o->h.magic != KNH_OBJECT_MAGIC) continue;
+//			if(o->h.refc == 0) {
+//				knh_Object_free(ctx, o);
+//				continue;
+//			}
+			knh_Object_setGCMarked(o, 1);
+		}
 	}
-}
-
-/* ------------------------------------------------------------------------ */
-
-void knh_Object_sweep1(Ctx *ctx, Object *o)
-{
-	if(knh_Object_isGCMarked(o)) {
-		knh_Object_traverse(ctx, o, knh_Object_sweep1);
+	DBG2_P("***** marking .. ******");
+	knh_Context_setMarking(ctx, 1);
+	konoha_traverse(ctx, knh_Object_mark0);
+	knh_Context_setMarking(ctx, 0);
+	DBG2_P("***** sweeping .. ******");
+	size_t c = 0;
+	knh_ftraverse fsweep = ctx->fsweep;
+	((Context*)ctx)->fsweep = knh_Object_finalSweep;
+	for(i = 0; i < size; i++) {
+		Object *t = ctx->tObjectTable[i].table;
+		for(j = 0; j < KNH_TOBJECT_SIZE; j++) {
+			Object *o = &t[j];
+			if(o->h.magic != KNH_OBJECT_MAGIC) continue;
+			if(knh_Object_isGCMarked(o)) {
+				//DBG2_P("%p: %s : refc=%d", o, CLASSN(o->h.cid), o->h.refc);
+#ifdef KNH_HOBJECT_REFC
+				o->h.refc = 0;
+#endif
+				knh_Object_free(ctx, o);
+				c++;
+			}
+		}
 	}
+	((Context*)ctx)->fsweep = fsweep;
+	DBG2_P("**** collected %d object(s) ****", (int)c);
 }
 
 /* ------------------------------------------------------------------------ */
@@ -516,6 +545,8 @@ knh_ftraverse konoha_getDefaultSweepFunction()
 	return knh_Object_RCsweep;
 }
 
+
+
 ///* ------------------------------------------------------------------------ */
 //
 //#define P11 ((knh_uint32_t)3)
