@@ -138,7 +138,7 @@ void konoha_initContext(Ctx *ctx, knh_Context_t *o, size_t stacksize)
 
 	DP(o->sys)->ctxcount += 1;
 	o->flag  = FLAG_CTX(0);
-	o->stacksize = (stacksize < 64) ? KNH_STACKSIZE : stacksize;
+	o->stacksize = (stacksize < 64) ? KONOHA_STACKSIZE : stacksize;
 	o->stack = (knh_sfp_t*)KNH_MALLOC(ctx, sizeof(knh_sfp_t) * o->stacksize);
 	o->esp = o->stack;
 	{
@@ -559,7 +559,7 @@ KNHAPI(konoha_t) konoha_open(size_t ctxsize, size_t stacksize)
 {
 	konoha_t k = {KONOHA_MAGIC};
 	if(ctxsize == 0) ctxsize = 1;
-	if(stacksize < KNH_STACKSIZE) stacksize = KNH_STACKSIZE;
+	if(stacksize < KONOHA_STACKSIZE) stacksize = KONOHA_STACKSIZE;
 	Ctx *ctx = konoha_initContextTable(ctxsize, 1024);
 	((Context*)ctx)->stacksize = stacksize; /* not beautiful */
 	konoha_initSystemTable((Context*)ctx);
